@@ -1,4 +1,3 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -783,7 +782,7 @@ int ParseFTPList(const char *line, struct list_state *state,
         state->parsed_one = 1;
         state->lstyle = lstyle;
 
-        p = &(line[linelen_sans_wsp]); /* line end sans wsp */
+        p = &(line[linelen]); /* line end */
         result->fe_cinfs = 1;
         result->fe_fname = tokens[3];
         result->fe_fnlen = p - tokens[3];
@@ -791,6 +790,12 @@ int ParseFTPList(const char *line, struct list_state *state,
 
         if (*tokens[2] != '<') /* not <DIR> or <JUNCTION> */
         {
+          // try to handle correctly spaces at the beginning of the filename
+          // filesize (token[2]) must end at offset 38
+          if (tokens[2] + toklen[2] - line == 38) {
+            result->fe_fname = &(line[39]);
+            result->fe_fnlen = p - result->fe_fname;
+          }
           result->fe_type = 'f';
           pos = toklen[2];
           while (pos > (sizeof(result->fe_size)-1))
@@ -798,37 +803,53 @@ int ParseFTPList(const char *line, struct list_state *state,
           memcpy( result->fe_size, tokens[2], pos );
           result->fe_size[pos] = '\0';
         }
-        else if ((tokens[2][1]) != 'D') /* not <DIR> */
-        {
-          result->fe_type = '?'; /* unknown until junc for sure */
-          if (result->fe_fnlen > 4)
+        else {
+          // try to handle correctly spaces at the beginning of the filename
+          // token[2] must begin at offset 24, the length is 5 or 10
+          // token[3] must begin at offset 39 or higher
+          if (tokens[2] - line == 24 && (toklen[2] == 5 || toklen[2] == 10) &&
+              tokens[3] - line >= 39) {
+            result->fe_fname = &(line[39]);
+            result->fe_fnlen = p - result->fe_fname;
+          }
+
+          if ((tokens[2][1]) != 'D') /* not <DIR> */
           {
-            p = result->fe_fname;
-            for (pos = result->fe_fnlen - 4; pos > 0; pos--)
+            result->fe_type = '?'; /* unknown until junc for sure */
+            if (result->fe_fnlen > 4)
             {
-              if (p[0] == ' ' && p[3] == ' ' && p[2] == '>' &&
-                  (p[1] == '=' || p[1] == '-'))
+              p = result->fe_fname;
+              for (pos = result->fe_fnlen - 4; pos > 0; pos--)
               {
-                result->fe_type = 'l';
-                result->fe_fnlen = p - result->fe_fname;
-                result->fe_lname = p + 4;
-                result->fe_lnlen = &(line[linelen_sans_wsp]) 
-                                   - result->fe_lname;
-                break;
+                if (p[0] == ' ' && p[3] == ' ' && p[2] == '>' &&
+                    (p[1] == '=' || p[1] == '-'))
+                {
+                  result->fe_type = 'l';
+                  result->fe_fnlen = p - result->fe_fname;
+                  result->fe_lname = p + 4;
+                  result->fe_lnlen = &(line[linelen]) 
+                                     - result->fe_lname;
+                  break;
+                }
+                p++;
               }
-              p++;
-            }    
+            }
           }
         }
-      
+
         result->fe_time.tm_mon = atoi(tokens[0]+0);
         if (result->fe_time.tm_mon != 0)
         {
           result->fe_time.tm_mon--;
           result->fe_time.tm_mday = atoi(tokens[0]+3);
           result->fe_time.tm_year = atoi(tokens[0]+6);
+          /* if year has only two digits then assume that
+               00-79 is 2000-2079
+               80-99 is 1980-1999 */
           if (result->fe_time.tm_year < 80)
-            result->fe_time.tm_year += 100;
+            result->fe_time.tm_year += 2000;
+          else if (result->fe_time.tm_year < 100)
+            result->fe_time.tm_year += 1900;
         }
 
         result->fe_time.tm_hour = atoi(tokens[1]+0);
@@ -983,6 +1004,8 @@ int ParseFTPList(const char *line, struct list_state *state,
        * "drwxr-xr-x  2 0  0  512 May 28 22:17 etc"
       */
     
+      gboolean is_old_Hellsoft = FALSE;
+
       if (numtoks >= 6)
       {
         /* there are two perm formats (Hellsoft/NetWare and *IX strmode(3)).
@@ -1008,6 +1031,8 @@ int ParseFTPList(const char *line, struct list_state *state,
             {
               /* rest is FMA[S] or AFM[S] */
               lstyle = 'U'; /* very likely one of the NetWare servers */
+              if (toklen[0] == 10)
+                is_old_Hellsoft = TRUE;
             }
           }
         }
@@ -1156,24 +1181,49 @@ int ParseFTPList(const char *line, struct list_state *state,
        
         } /* time/year */
         
-        result->fe_fname = tokens[tokmarker+4];
-        result->fe_fnlen = (&(line[linelen_sans_wsp]))
+        // there is exacly 1 space between filename and previous token in all
+        // outputs except old Hellsoft
+        if (!is_old_Hellsoft)
+          result->fe_fname = tokens[tokmarker+3] + toklen[tokmarker+3] + 1;
+        else
+          result->fe_fname = tokens[tokmarker+4];
+
+        result->fe_fnlen = (&(line[linelen]))
                            - (result->fe_fname);
 
         if (result->fe_type == 'l' && result->fe_fnlen > 4)
         {
-          p = result->fe_fname + 1;
-          for (pos = 1; pos < (result->fe_fnlen - 4); pos++)
+          /* First try to use result->fe_size to find " -> " sequence.
+             This can give proper result for cases like "aaa -> bbb -> ccc". */
+          guint32 fe_size = atoi(result->fe_size);
+
+          if (result->fe_fnlen > (fe_size + 4) &&
+              strncmp(result->fe_fname + result->fe_fnlen - fe_size - 4 , " -> ", 4) == 0)
           {
-            if (*p == ' ' && p[1] == '-' && p[2] == '>' && p[3] == ' ')
+            result->fe_lname = result->fe_fname + (result->fe_fnlen - fe_size);
+            result->fe_lnlen = (&(line[linelen])) - (result->fe_lname);
+            result->fe_fnlen -= fe_size + 4;
+          }
+          else
+          {
+            /* Search for sequence " -> " from the end for case when there are
+               more occurrences. F.e. if ftpd returns "a -> b -> c" assume
+               "a -> b" as a name. Powerusers can remove unnecessary parts
+               manually but there is no way to follow the link when some
+               essential part is missing. */
+            p = result->fe_fname + (result->fe_fnlen - 5);
+            for (pos = (result->fe_fnlen - 5); pos > 0; pos--)
             {
-              result->fe_lname = p + 4;
-              result->fe_lnlen = (&(line[linelen_sans_wsp]))
-                               - (result->fe_lname);
-              result->fe_fnlen = pos;
-              break;
+              if (p && g_str_has_prefix(p, " -> "))
+              {
+                result->fe_lname = p + 4;
+                result->fe_lnlen = (&(line[linelen]))
+                                 - (result->fe_lname);
+                result->fe_fnlen = pos;
+                break;
+              }
+              p--;
             }
-            p++;
           }
         }
 
