@@ -40,6 +40,8 @@
 #include <gvfsdaemonprotocol.h>
 #include <gvfsdaemonutils.h>
 #include <gvfsjobmount.h>
+#include <gvfsjobopenforread.h>
+#include <gvfsjobopenforwrite.h>
 #include <gdbusutils.h>
 
 enum {
@@ -997,10 +999,25 @@ peer_to_peer_filter_func (DBusConnection *conn,
 			  DBusMessage    *message,
 			  gpointer        data)
 {
+  GVfsDaemon *daemon = data;
+
   if (dbus_message_is_signal (message,
 			      DBUS_INTERFACE_LOCAL,
 			      "Disconnected"))
     {
+      GList *l;
+
+      g_mutex_lock (daemon->lock);
+      for (l = daemon->jobs; l != NULL; l = l->next)
+        {
+          GVfsJob *job = l->data;
+          
+          if (G_VFS_IS_JOB_DBUS (job) &&
+              G_VFS_JOB_DBUS (job)->connection == conn)
+            g_vfs_job_cancel (job);
+        }
+      g_mutex_unlock (daemon->lock);
+
       /* The peer-to-peer connection was disconnected */
       dbus_connection_unref (conn);
       return DBUS_HANDLER_RESULT_HANDLED;
@@ -1055,7 +1072,7 @@ g_vfs_daemon_initiate_mount (GVfsDaemon *daemon,
   obj_path = g_strdup_printf ("/org/gtk/vfs/mount/%d", ++daemon->mount_counter);
   backend = g_object_new (backend_type,
 			  "daemon", daemon,
-			  "object_path", obj_path,
+			  "object-path", obj_path,
 			  NULL);
   g_free (obj_path);
 
@@ -1065,3 +1082,33 @@ g_vfs_daemon_initiate_mount (GVfsDaemon *daemon,
   job = g_vfs_job_mount_new (mount_spec, mount_source, is_automount, request, backend);
   g_vfs_daemon_queue_job (daemon, job);
 }
+
+/**
+ * g_vfs_daemon_get_blocking_processes:
+ * @daemon: A #GVfsDaemon.
+ *
+ * Gets all processes that blocks unmounting, e.g. processes with open
+ * file handles.
+ *
+ * Returns: An array of #GPid. Free with g_array_unref().
+ */
+GArray *
+g_vfs_daemon_get_blocking_processes (GVfsDaemon *daemon)
+{
+  GArray *processes;
+  GList *l;
+
+  processes = g_array_new (FALSE, FALSE, sizeof (GPid));
+  for (l = daemon->job_sources; l != NULL; l = l->next)
+    {
+      if (G_VFS_IS_CHANNEL (l->data))
+        {
+          GPid pid;
+          pid = g_vfs_channel_get_actual_consumer (G_VFS_CHANNEL (l->data));
+          g_array_append_val (processes, pid);
+        }
+    }
+
+  return processes;
+}
+
