@@ -304,7 +304,10 @@ update_volume (GGduVolume *volume)
       volume->icon = gdu_presentable_get_icon (GDU_PRESENTABLE (volume->gdu_volume));
 
       g_free (volume->name);
-      volume->name = gdu_presentable_get_name (GDU_PRESENTABLE (volume->gdu_volume));
+      if (_is_pc_floppy_drive (device))
+        volume->name = g_strdup (_("Floppy Disk"));
+      else
+        volume->name = gdu_presentable_get_name (GDU_PRESENTABLE (volume->gdu_volume));
 
       /* special case the name and icon for audio discs */
       activation_uri = volume->activation_root != NULL ? g_file_get_uri (volume->activation_root) : NULL;
@@ -334,51 +337,12 @@ update_volume (GGduVolume *volume)
             volume->should_automount = FALSE;
         }
 
-      /* Avoid automounting volumes from
-       *
-       * 1. drives on a 'virtual' or unset bus
-       * 2. volumes without a drive
-       *
-       * This is to avoid interference with things like Fedora's
-       * livecd-tools that use device-mapper to set up images. See
-       * https://bugzilla.redhat.com/show_bug.cgi?id=494144 for more
-       * background.
-       *
-       * In the future we might want to relax 1. so automounting
-       * things like Linux MD and LVM2 devices work.
+      /* Respect the presentation hint for whether the volume should be automounted - normally
+       * nopolicy is only FALSE for "physical" devices - e.g. only "physical" devices will
+       * be set to be automounted.
        */
-      if (volume->drive != NULL)
-        {
-          GduPresentable *drive_presentable;
-          drive_presentable = g_gdu_drive_get_presentable (volume->drive);
-          if (drive_presentable != NULL)
-            {
-              GduDevice *drive_device;
-              drive_device = gdu_presentable_get_device (drive_presentable);
-              if (drive_device != NULL)
-                {
-                  const gchar *bus;
-                  bus = gdu_device_drive_get_connection_interface (drive_device);
-                  if (bus == NULL || strlen (bus) == 0 || g_strcmp0 (bus, "virtual") == 0)
-                    {
-                      volume->should_automount = FALSE;
-                    }
-                  g_object_unref (drive_device);
-                }
-              else
-                {
-                  volume->should_automount = FALSE;
-                }
-            }
-          else
-            {
-              volume->should_automount = FALSE;
-            }
-        }
-      else
-        {
-          volume->should_automount = FALSE;
-        }
+      if (gdu_device_get_presentation_nopolicy (device))
+        volume->should_automount = FALSE;
 
       g_free (activation_uri);
     }
@@ -647,22 +611,28 @@ g_gdu_volume_get_mount (GVolume *volume)
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gchar **
-get_mount_options (GduDevice *device)
+get_mount_options (GduDevice *device,
+                   gboolean   allow_user_interaction)
 {
-  gchar **ret;
+  GPtrArray *p;
+
+  p = g_ptr_array_new ();
 
   /* one day we might read this from user settings */
   if (g_strcmp0 (gdu_device_id_get_usage (device), "filesystem") == 0 &&
       g_strcmp0 (gdu_device_id_get_type (device), "vfat") == 0)
     {
-      ret = g_new0 (gchar *, 2);
-      ret[0] = g_strdup ("flush");
-      ret[1] = NULL;
+      g_ptr_array_add (p, g_strdup ("flush"));
     }
-  else
-    ret = NULL;
 
-  return ret;
+  if (!allow_user_interaction)
+    {
+      g_ptr_array_add (p, g_strdup ("auth_no_user_interaction"));
+    }
+
+  g_ptr_array_add (p, NULL);
+
+  return (gchar **) g_ptr_array_free (p, FALSE);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -784,7 +754,7 @@ mount_cleartext_device (MountOpData *data,
   else
     {
       gchar **mount_options;
-      mount_options = get_mount_options (data->device_to_mount);
+      mount_options = get_mount_options (data->device_to_mount, data->mount_operation != NULL);
       gdu_device_op_filesystem_mount (data->device_to_mount, mount_options, mount_cb, data);
       g_strfreev (mount_options);
     }
@@ -1012,10 +982,6 @@ mount_with_mount_operation (MountOpData *data)
   if (toplevel != NULL)
     drive_name = gdu_presentable_get_name (toplevel);
 
-  /* This is going to look ass until bug 573416 is fixed. Unfortunately
-   * the gtk+ maintain has stated "oh, I stopped using luks" but that's
-   * more of a gtk+ problem ;-)
-   */
   if (drive_name != NULL)
     {
       if (gdu_device_is_partition (device))
@@ -1206,7 +1172,7 @@ g_gdu_volume_mount (GVolume             *_volume,
     {
       gchar **mount_options;
       data->device_to_mount = g_object_ref (device);
-      mount_options = get_mount_options (data->device_to_mount);
+      mount_options = get_mount_options (data->device_to_mount, data->mount_operation != NULL);
       gdu_device_op_filesystem_mount (data->device_to_mount, mount_options, mount_cb, data);
       g_strfreev (mount_options);
     }
@@ -1491,11 +1457,12 @@ eject_wrapper_callback (GObject *source_object,
 }
 
 static void
-g_gdu_volume_eject (GVolume              *volume,
-                    GMountUnmountFlags   flags,
-                    GCancellable        *cancellable,
-                    GAsyncReadyCallback  callback,
-                    gpointer             user_data)
+g_gdu_volume_eject_with_operation (GVolume              *volume,
+                                   GMountUnmountFlags   flags,
+                                   GMountOperation     *mount_operation,
+                                   GCancellable        *cancellable,
+                                   GAsyncReadyCallback  callback,
+                                   gpointer             user_data)
 {
   GGduVolume *gdu_volume = G_GDU_VOLUME (volume);
   GGduDrive *drive;
@@ -1511,7 +1478,7 @@ g_gdu_volume_eject (GVolume              *volume,
       data->object = g_object_ref (volume);
       data->callback = callback;
       data->user_data = user_data;
-      g_drive_eject (G_DRIVE (drive), flags, cancellable, eject_wrapper_callback, data);
+      g_drive_eject_with_operation (G_DRIVE (drive), flags, mount_operation, cancellable, eject_wrapper_callback, data);
       g_object_unref (drive);
     }
   else
@@ -1529,9 +1496,9 @@ g_gdu_volume_eject (GVolume              *volume,
 }
 
 static gboolean
-g_gdu_volume_eject_finish (GVolume        *volume,
-                           GAsyncResult  *result,
-                           GError       **error)
+g_gdu_volume_eject_with_operation_finish (GVolume        *volume,
+                                          GAsyncResult  *result,
+                                          GError       **error)
 {
   GGduVolume *gdu_volume = G_GDU_VOLUME (volume);
   gboolean res;
@@ -1539,7 +1506,7 @@ g_gdu_volume_eject_finish (GVolume        *volume,
   res = TRUE;
   if (gdu_volume->drive != NULL)
     {
-      res = g_drive_eject_finish (G_DRIVE (gdu_volume->drive), result, error);
+      res = g_drive_eject_with_operation_finish (G_DRIVE (gdu_volume->drive), result, error);
     }
   else
     {
@@ -1550,6 +1517,24 @@ g_gdu_volume_eject_finish (GVolume        *volume,
   return res;
 }
 
+static void
+g_gdu_volume_eject (GVolume              *volume,
+                    GMountUnmountFlags   flags,
+                    GCancellable        *cancellable,
+                    GAsyncReadyCallback  callback,
+                    gpointer             user_data)
+{
+  g_gdu_volume_eject_with_operation (volume, flags, NULL, cancellable, callback, user_data);
+}
+
+static gboolean
+g_gdu_volume_eject_finish (GVolume        *volume,
+                           GAsyncResult  *result,
+                           GError       **error)
+{
+  return g_gdu_volume_eject_with_operation_finish (volume, result, error);
+}
+
 static char *
 g_gdu_volume_get_identifier (GVolume     *_volume,
                              const char  *kind)
@@ -1634,6 +1619,8 @@ g_gdu_volume_volume_iface_init (GVolumeIface *iface)
   iface->mount_finish = g_gdu_volume_mount_finish;
   iface->eject = g_gdu_volume_eject;
   iface->eject_finish = g_gdu_volume_eject_finish;
+  iface->eject_with_operation = g_gdu_volume_eject_with_operation;
+  iface->eject_with_operation_finish = g_gdu_volume_eject_with_operation_finish;
   iface->get_identifier = g_gdu_volume_get_identifier;
   iface->enumerate_identifiers = g_gdu_volume_enumerate_identifiers;
   iface->get_activation_root = g_gdu_volume_get_activation_root;
